"use strict";
/*
 * Copyright (c) 2021, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.OrgAccessor = exports.BaseOrgAccessor = exports.GlobalInfoOrgAccessor = void 0;
const fs = require("fs");
const path = require("path");
const kit_1 = require("@salesforce/kit");
const authInfoConfig_1 = require("../../config/authInfoConfig");
const global_1 = require("../../global");
const types_1 = require("../types");
const logger_1 = require("../../logger");
/**
 * @deprecated
 */
class GlobalInfoOrgAccessor {
    constructor(globalInfo) {
        this.globalInfo = globalInfo;
    }
    getAll(decrypt = false) {
        return this.globalInfo.get(types_1.SfInfoKeys.ORGS, decrypt);
    }
    get(username, decrypt = false) {
        const auth = this.globalInfo.get(`${types_1.SfInfoKeys.ORGS}["${username}"]`, decrypt);
        // For legacy, some things wants the username in the returned auth info.
        if (auth && !auth.username)
            auth.username = username;
        return auth;
    }
    has(username) {
        return !!this.getAll()[username];
    }
    set(username, org) {
        var _a;
        // For legacy, and to keep things standard, some things wants the username in auth info.
        (_a = org.username) !== null && _a !== void 0 ? _a : (org.username = username);
        this.globalInfo.set(`${types_1.SfInfoKeys.ORGS}["${username}"]`, org);
    }
    update(username, org) {
        var _a;
        // For legacy, and to keep things standard, some things wants the username in auth info.
        (_a = org.username) !== null && _a !== void 0 ? _a : (org.username = username);
        this.globalInfo.update(`${types_1.SfInfoKeys.ORGS}["${username}"]`, org);
    }
    unset(username) {
        delete this.globalInfo.get(types_1.SfInfoKeys.ORGS)[username];
    }
}
exports.GlobalInfoOrgAccessor = GlobalInfoOrgAccessor;
function chunk(array, chunkSize) {
    const final = [];
    for (let i = 0, len = array.length; i < len; i += chunkSize)
        final.push(array.slice(i, i + chunkSize));
    return final;
}
class BaseOrgAccessor extends kit_1.AsyncOptionalCreatable {
    constructor() {
        super(...arguments);
        this.configs = new Map();
        this.contents = new Map();
    }
    async read(username, decrypt = false, throwOnNotFound = true) {
        try {
            const config = await this.initAuthFile(username, throwOnNotFound);
            this.configs.set(username, config);
            return this.get(username, decrypt);
        }
        catch (err) {
            return null;
        }
    }
    async readAll(decrypt = false) {
        const fileChunks = chunk(await this.getAllFiles(), 50);
        for (const fileChunk of fileChunks) {
            const promises = fileChunk.map(async (f) => {
                const username = this.parseUsername(f);
                const config = await this.initAuthFile(username);
                this.configs.set(username, config);
            });
            await Promise.all(promises);
        }
        return this.getAll(decrypt);
    }
    get(username, decrypt = false) {
        const config = this.configs.get(username);
        if (config) {
            this.contents.set(username, config.getContents(decrypt));
        }
        return this.contents.get(username);
    }
    getAll(decrypt = false) {
        return [...this.configs.keys()].reduce((orgs, username) => {
            const org = this.get(username, decrypt);
            return org && !(0, kit_1.isEmpty)(org) ? orgs.concat([org]) : orgs;
        }, []);
    }
    has(username) {
        return this.contents.has(username);
    }
    async exists(username) {
        const config = this.configs.get(username);
        return config ? await config.exists() : false;
    }
    async stat(username) {
        const config = this.configs.get(username);
        return config ? await config.stat() : null;
    }
    async hasFile(username) {
        try {
            await fs.promises.access(this.parseFilename(username));
            return true;
        }
        catch {
            this.logger.debug(`No auth file found for ${username}`);
            return false;
        }
    }
    async list() {
        return this.getAllFiles();
    }
    set(username, org) {
        var _a, _b;
        const config = this.configs.get(username);
        if (config) {
            config.setContentsFromObject(org);
            const contents = config.getContents();
            (_a = contents.username) !== null && _a !== void 0 ? _a : (contents.username = username);
            this.contents.set(username, contents);
        }
        else {
            // @ts-ignore
            (_b = org.username) !== null && _b !== void 0 ? _b : (org.username = username);
            this.contents.set(username, org);
        }
    }
    update(username, org) {
        const existing = this.get(username) || {};
        const merged = Object.assign({}, existing, org);
        return this.set(username, merged);
    }
    async remove(username) {
        var _a;
        await ((_a = this.configs.get(username)) === null || _a === void 0 ? void 0 : _a.unlink());
        this.configs.delete(username);
        this.contents.delete(username);
    }
    async write(username) {
        const config = this.configs.get(username);
        if (config) {
            return (await config.write());
        }
        else {
            const contents = this.contents.get(username) || {};
            await this.read(username, false, false);
            // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
            const config = this.configs.get(username);
            config.setContentsFromObject(contents);
            return (await config.write());
        }
    }
    async init() {
        this.logger = await logger_1.Logger.child(this.constructor.name);
    }
    async getAllFiles() {
        const regex = this.getFileRegex();
        try {
            return (await fs.promises.readdir(global_1.Global.DIR)).filter((file) => regex.test(file));
        }
        catch {
            return [];
        }
    }
    parseUsername(filename) {
        return filename.replace(this.getFileExtension(), '');
    }
    parseFilename(username) {
        return path.join(global_1.Global.DIR, `${username}${this.getFileExtension()}`);
    }
}
exports.BaseOrgAccessor = BaseOrgAccessor;
class OrgAccessor extends BaseOrgAccessor {
    async initAuthFile(username, throwOnNotFound = false) {
        return authInfoConfig_1.AuthInfoConfig.create({
            ...authInfoConfig_1.AuthInfoConfig.getOptions(username),
            throwOnNotFound,
        });
    }
    getFileRegex() {
        // The regular expression that filters files stored in $HOME/.sfdx
        return /^[^.][^@]*@[^.]+(\.[^.\s]+)+\.json$/;
    }
    getFileExtension() {
        return '.json';
    }
}
exports.OrgAccessor = OrgAccessor;
//# sourceMappingURL=orgAccessor.js.map